import { EditorView, highlightSpecialChars, drawSelection, dropCursor, keymap } from '@codemirror/view';
import { Compartment, EditorSelection, EditorState } from '@codemirror/state';
import { HighlightStyle, syntaxHighlighting, indentOnInput, bracketMatching } from '@codemirror/language';
import { history, indentWithTab, defaultKeymap, historyKeymap } from '@codemirror/commands';
import { highlightSelectionMatches, searchKeymap } from '@codemirror/search';
import { snippet, closeBrackets, autocompletion, completionKeymap, closeBracketsKeymap } from '@codemirror/autocomplete';
import { lintKeymap } from '@codemirror/lint';
import { markdown, markdownLanguage } from '@codemirror/lang-markdown';
import { languages } from '@codemirror/language-data';
import { json } from '@codemirror/lang-json';
import { tags } from '@lezer/highlight';

function getMarkdownConfig() {
    return markdown({ codeLanguages: languages, base: markdownLanguage });
}
function getJsonConfig() {
    return json();
}
const LANGUAGE = new Compartment;

const config$1 = {
    name: 'yettoDark',
    dark: true,
    background: '#171717',
    foreground: '#fafafa',
    selection: '#3392FF44',
    cursor: '#c8e1ff',
    dropdownBackground: '#171717',
    dropdownBorder: '#1b1f23',
    activeLine: '#2b3036',
    matchingBracket: '#17E5E650',
    keyword: '#f97583',
    storage: '#f97583',
    variable: '#ffab70',
    parameter: '#e1e4e8',
    function: '#79b8ff',
    string: '#9ecbff',
    constant: '#79b8ff',
    type: '#79b8ff',
    class: '#b392f0',
    number: '#79b8ff',
    comment: '#6a737d',
    heading: '#79b8ff',
    invalid: '#f97583',
    regexp: '#9ecbff',
};
const yettoDarkTheme = EditorView.theme({
    '&': {
        color: config$1.foreground,
        backgroundColor: config$1.background,
    },
    '.cm-content': { caretColor: config$1.cursor },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: config$1.cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: config$1.selection },
    '.cm-panels': { backgroundColor: config$1.dropdownBackground, color: config$1.foreground },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: config$1.dropdownBackground,
        outline: `1px solid ${config$1.dropdownBorder}`
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: config$1.selection
    },
    '.cm-activeLine': { backgroundColor: config$1.activeLine },
    '.cm-selectionMatch': { backgroundColor: config$1.selection },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        backgroundColor: config$1.matchingBracket,
        outline: 'none'
    },
    '.cm-gutters': {
        backgroundColor: config$1.background,
        color: config$1.foreground,
        border: 'none'
    },
    '.cm-activeLineGutter': { backgroundColor: config$1.background },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: config$1.foreground
    },
    '.cm-tooltip': {
        border: `1px solid ${config$1.dropdownBorder}`,
        backgroundColor: config$1.dropdownBackground,
        color: config$1.foreground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent'
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: config$1.foreground,
        borderBottomColor: config$1.foreground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            background: config$1.selection,
            color: config$1.foreground,
        }
    }
}, { dark: config$1.dark });
const yettoDarkHighlightStyle = HighlightStyle.define([
    { tag: tags.keyword, color: config$1.keyword },
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: config$1.variable },
    { tag: [tags.propertyName], color: config$1.function },
    { tag: [tags.processingInstruction, tags.string, tags.inserted, tags.special(tags.string)], color: config$1.string },
    { tag: [tags.function(tags.variableName), tags.labelName], color: config$1.function },
    { tag: [tags.color, tags.constant(tags.name), tags.standard(tags.name)], color: config$1.constant },
    { tag: [tags.definition(tags.name), tags.separator], color: config$1.variable },
    { tag: [tags.className], color: config$1.class },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: config$1.number },
    { tag: [tags.typeName], color: config$1.type, fontStyle: config$1.type },
    { tag: [tags.operator, tags.operatorKeyword], color: config$1.keyword },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: config$1.regexp },
    { tag: [tags.meta, tags.comment], color: config$1.comment },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: config$1.heading },
    { tag: [tags.atom, tags.bool, tags.special(tags.variableName)], color: config$1.variable },
    { tag: tags.invalid, color: config$1.invalid },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
]);
const yettoDark = [
    yettoDarkTheme,
    syntaxHighlighting(yettoDarkHighlightStyle),
];

const config = {
    name: 'yettoLight',
    dark: false,
    background: '#fafafa',
    foreground: '#171717',
    selection: '#0366d625',
    cursor: '#044289',
    dropdownBackground: '#fafafa',
    dropdownBorder: '#e1e4e8',
    activeLine: '#f6f8fa',
    matchingBracket: '#34d05840',
    keyword: '#d73a49',
    storage: '#d73a49',
    variable: '#e36209',
    parameter: '#24292e',
    function: '#005cc5',
    string: '#032f62',
    constant: '#005cc5',
    type: '#005cc5',
    class: '#6f42c1',
    number: '#005cc5',
    comment: '#6a737d',
    heading: '#005cc5',
    invalid: '#cb2431',
    regexp: '#032f62',
};
const yettoLightTheme = EditorView.theme({
    '&': {
        color: config.foreground,
        backgroundColor: config.background,
    },
    '.cm-content': { caretColor: config.cursor },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: config.cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: config.selection },
    '.cm-panels': { backgroundColor: config.dropdownBackground, color: config.foreground },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        backgroundColor: config.dropdownBackground,
        outline: `1px solid ${config.dropdownBorder}`
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: config.selection
    },
    '.cm-activeLine': { backgroundColor: config.activeLine },
    '.cm-selectionMatch': { backgroundColor: config.selection },
    '&.cm-focused .cm-matchingBracket, &.cm-focused .cm-nonmatchingBracket': {
        backgroundColor: config.matchingBracket,
        outline: 'none'
    },
    '.cm-gutters': {
        backgroundColor: config.background,
        color: config.foreground,
        border: 'none'
    },
    '.cm-activeLineGutter': { backgroundColor: config.background },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: config.foreground
    },
    '.cm-tooltip': {
        border: `1px solid ${config.dropdownBorder}`,
        backgroundColor: config.dropdownBackground,
        color: config.foreground,
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent'
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: config.foreground,
        borderBottomColor: config.foreground,
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            background: config.selection,
            color: config.foreground,
        }
    }
}, { dark: config.dark });
const yettoLightHighlightStyle = HighlightStyle.define([
    { tag: tags.keyword, color: config.keyword },
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: config.variable },
    { tag: [tags.propertyName], color: config.function },
    { tag: [tags.processingInstruction, tags.string, tags.inserted, tags.special(tags.string)], color: config.string },
    { tag: [tags.function(tags.variableName), tags.labelName], color: config.function },
    { tag: [tags.color, tags.constant(tags.name), tags.standard(tags.name)], color: config.constant },
    { tag: [tags.definition(tags.name), tags.separator], color: config.variable },
    { tag: [tags.className], color: config.class },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: config.number },
    { tag: [tags.typeName], color: config.type, fontStyle: config.type },
    { tag: [tags.operator, tags.operatorKeyword], color: config.keyword },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: config.regexp },
    { tag: [tags.meta, tags.comment], color: config.comment },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: config.heading },
    { tag: [tags.atom, tags.bool, tags.special(tags.variableName)], color: config.variable },
    { tag: tags.invalid, color: config.invalid },
    { tag: tags.strikethrough, textDecoration: 'line-through' },
]);
const yettoLight = [
    yettoLightTheme,
    syntaxHighlighting(yettoLightHighlightStyle),
];

const THEME = new Compartment;
const toggleTheme = (view) => {
    const newTheme = THEME.get(view.state) == yettoLight ? yettoDark : yettoLight;
    view.dispatch({
        effects: [
            THEME.reconfigure(newTheme),
        ]
    });
    return true;
};

function prependLines(mark, view, ordered = false) {
    view.dispatch(view.state.changeByRange(range => {
        const changes = [];
        let firstLine = view.state.doc.lineAt(range.from).number;
        let lastLine = view.state.doc.lineAt(range.to).number;
        let listIdx = 1;
        let newHead = range.to;
        // Iterate over the lines
        for (let i = firstLine; i <= lastLine; i++, listIdx++) {
            let line = view.state.doc.line(i);
            let text = ordered ? `${listIdx}${mark} ` : `${mark} `;
            newHead += text.length;
            // Create a change that inserts the string at the start of the line
            changes.push({
                from: line.from,
                insert: text
            });
        }
        // TODO: Figure out how to get the head value correct after prepending
        return {
            range: EditorSelection.range(range.from, newHead),
            changes
        };
    }));
}
function makePrependLinesCommand(mark, ordered = false) {
    return (view) => {
        prependLines(mark, view, ordered);
        return true;
    };
}
const BulletedList = {
    key: "Ctrl-Shift-8",
    mac: "Cmd-Shift-8",
    run: makePrependLinesCommand("-"),
    preventDefault: true
};
const NumberedList = {
    key: "Ctrl-Shift-7",
    mac: "Cmd-Shift-7",
    run: makePrependLinesCommand(".", true),
    preventDefault: true
};
const TaskList = {
    key: "Ctrl-Shift-l",
    mac: "Meta-Shift-l",
    run: makePrependLinesCommand("- [ ]"),
    preventDefault: true
};
const QuoteText = {
    key: "Ctrl-Shift-.",
    mac: "Cmd-Shift-.",
    run: makePrependLinesCommand(">"),
    preventDefault: true
};

function wrapText(mark, view) {
    view.dispatch(view.state.changeByRange(range => ({
        changes: [{ from: range.from, insert: mark }, { from: range.to, insert: mark }],
        range: EditorSelection.range(range.from + mark.length, range.to + mark.length)
    })));
}
function makeWrapTextCommand(mark) {
    return (view) => {
        wrapText(mark, view);
        return true;
    };
}
const BoldText = {
    key: "Ctrl-b",
    mac: "Cmd-b",
    run: makeWrapTextCommand("**"),
    preventDefault: true,
};
const ItalicText = {
    key: "Ctrl-i",
    mac: "Cmd-i",
    run: makeWrapTextCommand("*"),
    preventDefault: true,
};
const CodeText = {
    key: "Ctrl-e",
    mac: "Cmd-e",
    run: makeWrapTextCommand("`"),
    preventDefault: true,
};

const options = [
    {
        label: "/image",
        type: "text",
        apply: snippet("![${1:alt_text}](${2:url})"),
        detail: "An inline image",
        section: "Markdown"
    },
    {
        label: "/link",
        type: "text",
        apply: snippet("[${1:link_text}](${2:url})"),
        detail: "A link to some other place",
        section: "Markdown"
    },
    {
        label: "/codeBlock",
        type: "text",
        apply: snippet("```${1:language}\n${2:code}\n```"),
        detail: "A block of code or other preformatted text",
        section: "Markdown"
    },
];
function markdownCompletions(context) {
    let word = context.matchBefore(/\/\w*/);
    if (!word)
        return;
    if (word.from == word.to && !context.explicit)
        return null;
    return {
        from: word.from,
        options,
    };
}

function getSthenoConfig(lang, extensions) {
    const language = lang === 'markdown' ? getMarkdownConfig() : getJsonConfig();
    const config = {
        extensions: [
            highlightSpecialChars(),
            history(),
            drawSelection(),
            dropCursor(),
            EditorState.allowMultipleSelections.of(true),
            indentOnInput(),
            bracketMatching(),
            closeBrackets(),
            autocompletion(),
            highlightSelectionMatches(),
            EditorView.lineWrapping,
            keymap.of([
                NumberedList,
                BulletedList,
                TaskList,
                QuoteText,
                BoldText,
                ItalicText,
                CodeText,
                indentWithTab,
                ...defaultKeymap,
                ...lintKeymap,
                ...completionKeymap,
                ...historyKeymap,
                ...searchKeymap,
                ...closeBracketsKeymap,
            ]),
            THEME.of(yettoLight),
            LANGUAGE.of(language),
            language.language.data.of({
                autocomplete: markdownCompletions
            })
        ],
    };
    return config;
}

export { BulletedList, NumberedList, QuoteText, THEME, TaskList, getSthenoConfig, makePrependLinesCommand, makeWrapTextCommand, prependLines, toggleTheme, wrapText, yettoDark, yettoLight };
